import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.TextField;
import javax.microedition.midlet.MIDletStateChangeException;



/**
 * @author Gregor Broll, Alex De Luca
 * 
 * This class implements a form that allows the input of arbitrary strings. It also implements Http-POST- und GET-connections to the server.
 */
public class InputForm extends Form implements CommandListener, Runnable
{
	private PMP3Midlet midlet;
	private RecordManager recordManager;
	private TextField inputField;
	private boolean useGet;								//switches between using a GET or POST request
	private String input;
	
	private final static Command sendCMD = new Command("Send", Command.OK, 1);
	private final static Command historyCMD = new Command("History", Command.ITEM, 2);
	private final static Command exitCMD = new Command("Exit", Command.EXIT, 1);
	
	
	/**
	 * Constructor; sets up the textfield for input
	 * 
	 * @param midlet
	 * @param recordManager
	 */
	public InputForm(PMP3Midlet midlet, RecordManager recordManager)
	{
		super("Input");
		this.midlet = midlet;
		this.recordManager = recordManager;
		useGet = false;
		input = "";
		
		inputField = new TextField("Please provide an input for the server-request", "", 100, TextField.ANY);
		append(inputField);
		
		setCommandListener(this);
		addCommand(sendCMD);
		addCommand(historyCMD);		
		addCommand(exitCMD);	
	}
	
	
	/**
	 * Command-handler
	 */
	public void commandAction(Command cmd, Displayable d) 
	{
		if(cmd == sendCMD)
		{
			input = inputField.getString();
			
			//checking the validity of the provided input
			if(input.equals(""))
			{
				midlet.showAlert("", "Please provide some input!", this);
			}
			else
			{
				//show a waitingform while connecting to the server
				midlet.showDisplayable(new WaitingForm(midlet, this));
				run();
			}
		}
		
		if(cmd == historyCMD)
		{
			midlet.showDisplayable(new RecordHistoryForm(midlet, recordManager));
		}
		
		if(cmd == exitCMD)
		{
			try 
			{
				midlet.destroyApp(true);
			} 
			catch (MIDletStateChangeException e) 
			{
				e.printStackTrace();
			}
		}
	}
		
	
	/**
	 * Implements POST- and GET-connections to the server
	 */
	public void run()
	{
		String url = "http://murx.medien.ifi.lmu.de/~gregor/echo.php";
		HttpConnection httpConn = null;
		InputStream is = null;		
		StringBuffer buffer = new StringBuffer();		
		
		try
        {	        
        	if(useGet == true)
        	{	        		
	        	httpConn = (HttpConnection)Connector.open(url + "?input=" + input);		   	       //GET is the default setting
	        	
	        	System.out.println("GET-request with url " + url + "?input=" + input + " sent");
        	}	        		
        	else if(useGet == false)
        	{
        		OutputStream os = null;
	        	String request = "input=" + input;		        	
	        	
	        	httpConn = (HttpConnection)Connector.open(url);	        	
	        	httpConn.setRequestMethod(HttpConnection.POST);			
	        	httpConn.setRequestProperty( "Content-Type", "application/x-www-form-urlencoded");		//identifies the data to be sent; key-value-pairs
	        	
	        	os = httpConn.openOutputStream();
	        	os.write(request.getBytes());

	        	os.flush();			//Flushes this output stream and forces any buffered output bytes to be written out.
	        	os.close();
	        	System.out.println("POST request with url " + url + " and request " + request + " sent");
        	}	           		        		

        	
            is = httpConn.openInputStream();	           
            int ch;

            while ((ch = is.read()) != -1)
            {
                buffer = buffer.append((char) ch);
            }

            is.close();   
            httpConn.close();
            
            recordManager.writeRecord(buffer.toString());
            inputField.setString("");
            midlet.showAlert("Server-Answer:", buffer.toString(), this);
        }
        catch (IOException e)
        {
        	e.printStackTrace();
        }
	}
}
